#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import re
import json

# Определяем тип тестирующей системы
# В отладочных целях можно заменить эти переменные на константы True/False
POLYGON = ("polygon" in os.environ.get("APPDATA", "0").lower()) or (
        "codeforces" in os.environ.get("APPDATA", "0").lower())
EJUDGE = "EJUDGE" in os.environ
SILVER = "SILVERTESTS_VERSION" in os.environ
SIRIUS = not (POLYGON or EJUDGE or SILVER)

# Коды возврата для передачи в функцию quit
OK = 0
WA = 1
PT = 1
PE = 2
CF = 3


# Класс для работы с ответом участника в тестирующей системе Сириуса
class AnswerSMT():
    def __init__(self, userAnswer=None, score=0, maxScore=100):
        '''считывание ответа'''
        if userAnswer is None:
            self.userAnswer = json.loads("".join(sys.stdin.readlines()))['solution']
        else:
            self.userAnswer = json.loads(userAnswer)

        '''считаем количество элементов ввода и делаем список ответов по элементам ввода'''
        if type(self.userAnswer) is dict:
            self.elements = 1
            if type(self.userAnswer['solution']) is dict:
                self.userAnswer = [[self.userAnswer['solution']['message']]]
            elif type(self.userAnswer['solution']) is int:
                self.userAnswer = [[self.userAnswer['solution']]]
            else:
                self.userAnswer = [self.userAnswer['solution']]
        else:
            self.elements = len(self.userAnswer)
            a = []
            for i in range(0, self.elements):
                if type(self.userAnswer[i]['solution']) is int:
                    a.append([self.userAnswer[i]['solution']])
                else:
                    a.append(self.userAnswer[i]['solution'])
            self.userAnswer = a

        '''считаем, сколько полей ввода для каждого элемента ввода'''
        self.userAnswerLength = []
        for i in range(0, self.elements):
            if type(self.userAnswer[i]) is list:
                self.userAnswerLength.append(len(self.userAnswer[i]))
            elif self.userAnswer[i] != '':
                self.userAnswerLength.append(1)
            else:
                self.userAnswerLength.append(0)

        self.comment = ''
        self.score = score
        self.maxScore = maxScore
        self.verdict = 'WA'

    def answer(self, element=0, field=0):
        '''Возвращает ответ участника из элмента ввода с номером element.
        Если в элементе ввода несколько полей ввода, то можно указать номер поля ввода'''
        if element >= self.elements:
            return None
        else:
            a = self.userAnswer[element]

        if a is None:
            return None
        if field >= len(a):
            return None
        else:
            return None if a[field] is None else str(a[field])

    def fields_in_element(self, element=0):
        '''Возвращает информацию о количестве полей ввода в элементе ввода'''
        if len(self.userAnswerLength) <= element:
            return 0
        else:
            return self.userAnswerLength[element]

    def print_answer(self):
        '''посмотреть всё содержимое для отладки'''
        print("user answer:")
        print(self.userAnswer)
        print("number of elements")
        print(self.elements)
        print('length of elements')
        print(self.userAnswerLength)

    def quit(self, comment=None, score=None, maxScore=None, verdict=None):
        if not comment is None:
            self.comment = comment

        if not score is None:
            self.score = score

        if not maxScore is None:
            self.maxScore = maxScore

        if not verdict is None:
            self.verdict = verdict

        print(json.dumps({"maxScore": self.maxScore,
                          "score": self.score,
                          "comment": self.comment,
                          "verdict": self.verdict}))
        sys.exit(0)


def quit(status, log="", score=0):
    if SIRIUS:
        if status in [OK, WA] and 0 < score < 100:
            status = PT
        comment = log.replace("\n", "\n\n")  # Дублируем концы строк для разметки
        sirius_ans_obj.quit(verdict=status, comment=comment, score=score)
        return

    if POLYGON:  # Нужно в Polygon, т.к. там Windows
        sys.stdout.reconfigure(encoding='utf-8')
        sys.stderr.reconfigure(encoding='utf-8')

    # Выводим набранные баллы
    if status in [OK, WA]:
        if POLYGON:
            sys.stderr.write("points " + str(score) + "\n")
            status = 7  # Код возврата полигона, сообщающий о наличии баллов
        elif EJUDGE or SILVER:
            sys.stdout.write(str(score) + "\n")

    # Выводим протокол проверки
    sys.stderr.write(log)

    # Возвращаем код возврата
    sys.exit(status)


def init():
    global OK, WA, PE, CF, PT
    global sirius_ans_obj
    if SIRIUS:
        PE = "PE"
        WA = "WA"
        OK = "OK"
        СF = "CF"
        PT = "PT"
        # При создании объекта можно в конструктор в целях отладки передать строку с JSON, тогда ответ будет считан с этого JSON, иначе из stdin
        sirius_ans_obj = AnswerSMT()


init()

########################################################################################
# Код дальше можно править
#
# Код этой функции надо исправить так, чтобы она возвращала ответ, полученный из JSON в Сириусе
# или полученный из файла в тестирующих системах Ejudge или Polygon
# Формат возвращаемого значения - произвольный, например, одна строка или список строк
# При необходимости можно прямо в этом коде делать quit(PE, ...)
#
# В тестирующей системе Сириуса:
# Задание состоит из "элементов" (разделов"), внутри раздела могут быть поля ввода.
# Например, задание вида "Дайте ответ для а) N = 10 б) N = 20, в) N = 30" - это три раздела, они будут визуально отделены.
# А задание вида "Напишите в ответе номера объектов, которые вы выбрали, добавляя поля по необходимости" - это один раздел с несколькими полями ввода
# При считывании данных из тестирующей системы Сириуса нужно использовать объект sirius_ans_obj
# со следующим интерфейсом.
# sirius_ans_obj.elements - количество "элементов" в ответе.
# sirius_ans_obj.fields_in_element(element=0) - возвращает количество полей ввода в элементе номер element, нумерация с нуля
# sirius_ans_obj.answer(element=0, field=0) - возвращает значение поля ввода номер field из элемента номер element, нумерация с нуля
########################################################################################


def get_ans():
    if SIRIUS:
        ans = []
        elem = sirius_ans_obj.answer(element=0)
        for line in elem.split():
            line = line.strip()
            if line:
                ans.append(line)
        return ans
        return ans
    elif POLYGON or SILVER:
        if POLYGON:
            fin = open(sys.argv[2], "r", encoding="utf8")
        elif SILVER:
            fin = sys.stdin
        ans = []
        for line in fin:
            line = line.strip()
            if line:
                ans.append(line)
        return ans

ans = get_ans()

for i in range(len(ans)):
    ans[i] = ans[i].strip().replace(",", "").replace(";", "").replace(" ", "")
    ans[i] = ans[i].replace(chr(8722), "-")
    ans[i] = ans[i].replace(chr(8212), "-")
    ans[i] = ans[i].replace(chr(8210), "-")
    ans[i] = ans[i].replace(chr(8211), "-")

for i in range(len(ans)):
    if ans[i] == "-":
        continue
    if len(ans[i]) > 2:
        quit(PE, f"В строке {i+1} ответа записано больше 2 символов. В каждой строке должен быть 1 или 2 символа")
    for c in ans[i]:
        if c not in "56789":
            quit(PE, f"Строка {i+1} ответа неверная. В каждой строке должны быть одна или две цифры 5, 6, 7, 8, 9 или знак -.")
        if len(ans[i]) == 2 and ans[i][0] == ans[i][1]:
            quit(PE, f"В строке {i+1} ответа записаны две одинаковые цифры. Если строка содержит две цифры, то они должны быть различными")

child = [set(), set()]
child[0] = {5, 6, 7, 8, 9}


for i in range(len(ans)):
    coast1 = i % 2
    coast2 = 1 - coast1
    if ans[i] != "-":
        for c in ans[i]:
            c = int(c)
            if c not in child[coast1]:
                quit(WA, f"Исполнение алгоритма невозможно, в строке {i+1} дана команда перемещения ребёнка возраста {c}, но его нет на этом берегу.", score=0)
            child[coast1].remove(c)
            child[coast2].add(c)
    for c in range(5, 9):
        if (c in child[coast1]) and ((c + 1) in child[coast1]):
            quit(WA, f"Исполнение строки {i+1} алгоритма привело к тому, что на берегу остались дети возраста {c} и {c+1}, они подерутся", score=0)

if child[0] != set():
    quit(WA, f"Алгоритм некорректен, после исполнения алгоритма на берегу остались дети {sorted(child[0])}", score=0)

if len(ans) % 2 == 0:
    quit(WA, "Алгоритм некорректен, после его исполнения мама оказалась на первоначальном берегу")

if len(ans) < 7:
    quit(WA, f"Ваш ответ лучше ответа жюри, вы можете подать апелляцию", score=1)

SCORES = {
        7: 100,
        9: 60,
        11: 30,
        13: 20,
}

score = SCORES.get(len(ans), 10)

if score == 100:
    quit(OK, "Это наилучший ответ", score=score)
else:
    quit(WA, "Алгоритм перемещения корректный, но не самый короткий", score=score)

